function onScriptLoad( )
{
	// Create/load the database
 	database <- ConnectSQL( "databases/racing.db" );

	// Bind the track data to the skins.
	dofile( "functions/bind.nut" );

	// Load the generic functions.
	dofile( "functions/generic.nut" );

	// Activate game and server events.
	dofile( "events/server.nut" );
	dofile( "events/race.nut" );

	// Load the stopwatch class.
	dofile( "classes/stopwatch.nut" );

	// Load each track from the database into the roottable.
 	LoadCheckpoints();

	print( "=== Loaded Squirrel Racing - By rulk ===" );
}

const StreamZone        =   90; 		// Checkpoint Streaming proximity.
const MyRadius          =   40;   		// IsPlayerAtCheckpoint radius.

const intMyPickups      =   15;			// Number of pickups to allocate each player for streaming in the StreamZone.
const MyMaxPlayers      =   20;			// 20x15 = 300 objects for streaming. ( vcmp object limit is 2000 )


function LoadCheckpoints()
{
	// Create a static array, to allocate each player 6 pickups for streaming.
	MyPickups <- array( MyMaxPlayers, null );

	//  Make this array multidimensional and place the 6 pickup instances into columns,
	//  allowing us to globally use all the properties for each pickup object.

	//  We will use the '.Pos' property to move, 'stream' each players column of pickups
	//  to the position of the checkpoint as they enter the proximity of it.

	//  view the wiki for more of the pickup properties.

	//  player1     player2     player3   ...
	//
	//  pickup1     pickup1     pickup1   ...
	//  pickup2     pickup2     pickup2   ...
	//  pickup3     pickup3     pickup3   ...
	//    ...         ...         ...

	// Call our custom function.
	CreatePickups();

	// Create a static array with all values starting at zero
	// to hold the last checkpoint number for each participant.
	MyLastCheckPoint <- array( MyMaxPlayers, 0 );

	// Create a static array, to hold the marker for each player.
	MyMarker <- array( MyMaxPlayers, 0 );

	// Create a global variable to hold the timer object, 
	// allowing us to toggle on/off streaming.
 	streaming <- NewTimer( "StreamCheckpoints", 450, 0 );

	// start a timer to check unoccupied vehicles remain at each grid. ( interval in milliseconds )
	restoregrid <- NewTimer( "RestoreGrid", 45000, 0 );

	// Query the database to count how many tracks are avilable, and store it in a new slot.
	Num_Tracks <-  GetSQLColumnData( QuerySQL( database, "SELECT COUNT(*) FROM sqlite_master WHERE type='table'"  ), 0 );

	// loop through the skins.
	for (local skinid=0; skinid < 200; skinid++)
	{
		// If there is a track associated with the skin,
		// load it into the roottable.
		if ( FindTrack(skinid) != null ) 
		{
			// Grab the track-name		
			local checkpoint = FindTrack(skinid);

			// Create a dynamic array, using the race-name as its identifer,
			// to store the checkpoints from the database.
			// the array index will represent the checkpoint number and
			// We will use index zero to store the starting grid.
			this[ checkpoint ] <- [ [] ];			

			// Query the database to retrieve each checkpoint coordinates as data type 'string'
			local q = QuerySQL( database, "SELECT * FROM '" + checkpoint + "'"  );

			// int to count rows
			local n = 1;

			// Iterate the rows in a loop until there are no more rows (column data = null)
			while ( GetSQLColumnData( q, 0 ) != null )
			{
				local
                    		    Axis 	=    split( GetSQLColumnData( q, 0 ), " " ),
                       		    x   	=    Axis[ 0 ].tofloat(),
                       		    y   	=    Axis[ 1 ].tofloat(),
		       		    z   	=    Axis[ 2 ].tofloat(),
		       		    angle 	=    GetSQLColumnData( q, 1 ),
		       		    IsGrid 	=    GetSQLColumnData( q, 2 );

				if ( IsGrid == "true" )
				{
					// Store the value in array index zero as data type 'Vector' ( for convenience later )
					this[checkpoint][ 0 ].insert( 0, Vector(x, y, z) );

					// Position vehicles at the starting grid axis, in their 'bound' world and 'bound' Vehicle Model/ID.
					CreateGrid( this[checkpoint][ 0 ][ 0 ], FindTrackWorld( checkpoint ), 145 );
				} 
				else if ( IsGrid  == "false" )
				{

					// place the coordinates for the 6 pickups that make up a checkpoint into columns in our array.
					//
					//     checkpoint1_Grid		  checkpoint2_Grid	    checkpoint3_Grid	      ...
					// checkpoint1_pickup.pos_1   checkpoint2_pickup.pos_1  checkpoint3_pickup.pos_1      ...
					// checkpoint1_pickup.pos_2   checkpoint2_pickup.pos_2 	checkpoint3_pickup.pos_2      ...
					// checkpoint1_pickup.pos_3   checkpoint2_pickup.pos_3 	checkpoint3_pickup.pos_3      ...
					//           ...			...			   ...

					// We will then move the player's 6 pickups to these coordinates when they enter the given 'StreamZone'.
					

					// Lets dynamically build the array next,
					// This way, our array will never be larger than needed.

					// push an empty element into the next slot, these are our columns.
					this[checkpoint].push( [] );

					// Fill the columns with the coordinates we want each of the 6 pickups, 
					// that represent a checkpoint to be placed.

					// Note: All calculations circle the centre pickup.
					//		o   o   o
  					//		o [ o ] o					
					//
					switch ( angle )
					{
						// horizontal
						case "H":
						this[checkpoint][ n ].push( Vector( x, y, z ) );
						this[checkpoint][ n ].push( Vector( x, y + 2, z ) );
						this[checkpoint][ n ].push( Vector( x, y - 2, z ) );
						this[checkpoint][ n ].push( Vector( x, y, z + 2 ) );
						this[checkpoint][ n ].push( Vector( x, y + 2, z + 2 ) );
						this[checkpoint][ n ].push( Vector( x, y - 2, z + 2 ) );
						break;

						// vertical
						case "V":
						this[checkpoint][ n ].push( Vector( x, y, z ) );
						this[checkpoint][ n ].push( Vector( x + 2, y, z ) );
						this[checkpoint][ n ].push( Vector( x - 2, y, z ) );
						this[checkpoint][ n ].push( Vector( x, y, z + 2 ) );
						this[checkpoint][ n ].push( Vector( x + 2, y, z + 2 ) );
						this[checkpoint][ n ].push( Vector( x - 2, y, z + 2 ) );
						break;

						// r-angled
						case "R":
						this[checkpoint][ n ].push( Vector( x, y, z ) );
						this[checkpoint][ n ].push( Vector( x + 2, y - 1.5, z ) );
						this[checkpoint][ n ].push( Vector( x - 2, y + 1.5, z ) );
						this[checkpoint][ n ].push( Vector( x, y, z + 2 ) );
						this[checkpoint][ n ].push( Vector( x + 2, y - 1.5, z + 2 ) );
						this[checkpoint][ n ].push( Vector( x - 2, y + 1.5, z + 2 ) );
						break;

						// l-angled
						case "L":
						this[checkpoint][ n ].push( Vector( x, y, z ) );
						this[checkpoint][ n ].push( Vector( x + 1.5, y + 2, z ) );
						this[checkpoint][ n ].push( Vector( x - 1.5, y - 2, z ) );
						this[checkpoint][ n ].push( Vector( x, y, z + 2 ) );
						this[checkpoint][ n ].push( Vector( x + 1.5, y + 2, z + 2 ) );
						this[checkpoint][ n ].push( Vector( x - 1.5, y - 2, z + 2 ) );
						break;
					}
				}

     				// Read the next row available
     				GetSQLNextRow( q );

				// Increase row count
	   			n++;

				// Terminate the loop when no rows are avilable.
				if ( !q ) break;
			}

			// Free the query, we've done for now.
			FreeSQLQuery( q );
		}
	}
}


function CreatePickups()
{
	// construct a static 10x6 two-dimensional array.
	//
	// 	10 = the number of players.
	// 	6  = the number of pickups per player.
	//
	//	Note: This has now been changed to 10x12 2d array, 
        //            in order to stream two checkpoints to each player.
	// 	      when they are in the given 'StreamZone'.
	//      
	//	Note: You can now influance this array using
	//	      the constants at the top.

	try {
	
		for (local i=0; i < MyPickups.len(); i++) 
		{
			MyPickups[ i ] = array( intMyPickups, null );
		}

		// Loop through the number of rows in the array.
		for (local row=0; row < MyPickups.len(); row++)
		{
			// Loop through the columns in the rowth row.
			for (local col=0; col < MyPickups[row].len(); col++)
			{
				// Create the pickup at position 0,0,0
				// and store it's instance in the array index for use later.

				// Syntax: CreatePickup( int model, int world, int quantity, Vector * pos, int alpha, bool isAuto );
				MyPickups[row][col] = CreatePickup( 382, row + 1, 0, Vector( 0, 0, 0 ), 255, false );

				// We will set the world parameter to the rowid + 1. ( Pickup.World = player.ID + 1 )

				// This means that the Pickups for Player0 are in world1,
				// The pickups for Player1 are in World2 and the pickups for
				// Player2 are in World3 etc etc etc.... 
				// we have to do this because world0 is for everyone.
			}
		}

    	} catch( e );
}

function CreateGrid(axis, world, vehicle)
{
	local
            x = axis.x,
            y = axis.y, 
            z = axis.z; // Vector floats

	local a = 1, b = 1, pos;

	for ( local i = 0; i < 8; i++ )
	{
		// increase vars
		a += 4.6, b++;

		// Establish IsEven status and
		// set the vehicle position accordingly.
		if ( b % 2 != 0 ) pos = Vector(x, y - a, z);
		else pos = Vector(x - 4.6, y - a, z);

		// Create the vehicles - FindTrackWorld() is where we specify the world for each track.
		CreateVehicle( vehicle, world, pos, 100.5, 1, 1 );
	}
}


function StreamCheckpoints()
{
	// Iterate all online players
	for ( local r = 0; r < 50; r++ )
	{
		// Find the players name		
		local player = FindPlayer( r );

		// If we have at least one player is online.
		if ( player )
		{
			// loop through the skins.
			for (local skinid=0; skinid < 200; skinid++)
			{
				// If there is a track associated with the skin,
				// and it exists as an array in the roottable.
				if ( ( FindTrack(skinid) != null ) && ( FindTrack(skinid) in this && typeof this[ FindTrack(skinid) ] == "array") )
				{
					// Grab the array name.
					local checkpoint = FindTrack(skinid);

					// int to count each players pickups.
					local n = 0;

					// Loop through the number of rows in the array.
					for (local row=1; row < this[checkpoint].len(); row++) 
					{
						// Loop through the columns in the rowth row.
						for (local col=0; col < this[checkpoint][row].len(); col++)
						{
							// Ensure the player has a matching skin, and check the player's distance is within the 'StreamZone'
							if  ( ( player.Skin == skinid ) )
							{
								// Establish the player's distance from checkpoint
								local distance = DistanceFromPoint( player.Pos.x, player.Pos.y, this[checkpoint][ row ][ col ].x, this[checkpoint][ row ][ col ].y );
								
								// Make sure it's within our StreamZone
								if ( distance <= StreamZone )
								{
									// reset our counter when it reaches each players object limit.
									if ( n % MyPickups[ player.ID ].len() == 0 ) n = 0;

									// Move the player's column of pickups to the matched coordinates.
									MyPickups[ player.ID ][ n ].Pos = this[checkpoint][ row ][ col ];

									// Set the pickup World Property.
									MyPickups[ player.ID ][ n ].World = player.UniqueWorld;

									// increase the player's pickup counter
									n++;
								}
							}

						}
					}					
				}
			}
		}
	}
}


function IsPlayerAtCheckpoint(player)
{
	// Establish which track the player is on.
	local checkpoint = FindTrack( player.Skin );

	// Iterate through the checkpoint array.
	for ( local i = 1; i < this[checkpoint].len(); i++ )
	{
		// Ensure the index has a value, and check the player's distance is within the specified radius. ( paramater two )
		if ( ( this[checkpoint][ i ][ 0 ] != null ) )
		{
			// Establish the player's distance from checkpoint
			local distance = DistanceFromPoint( player.Pos.x, player.Pos.y, this[checkpoint][ i ][ 0 ].x, this[checkpoint][ i ][ 0 ].y );

			// Make sure it's within our Radius
			if ( distance <= MyRadius )
			{
				// once the above is established, we can now ensure the player collects
				// all checkpoints in sucession.

				// we can do this simply by using the difference between the
				// current checkpoint number and the Last checkpoint the player
				// collected.

				// This working example confirms the checkpoint number is greater than the last...
				// and the difference betewen the current checkpoint number
				// and the last checkpoint collected, is one.

				if ( ( MyLastCheckPoint[ player.ID ] < i ) && ( ( i - MyLastCheckPoint[ player.ID ] ) == 1 ) )
				{
					// Destroy all previous markers first.
					RemoveMarkers( player );

					// Now, Create a marker at the next checkpoint position.
					try{
						MyMarker[ player.ID ] = CreateMarker( player.UniqueWorld, this[checkpoint][ i + 1 ][ 0 ], 5, RGB(0, 0, 0), 1 );
					} catch ( e );

					// store the checkpoint number in a seperate array for use later.
					MyLastCheckPoint[ player.ID ] = i;

					// Return the checkpoint number for output in-game.
					return i;				
				}

				// Anything else is a missed checkpoint.
				else if ( MyLastCheckPoint[ player.ID ] != i ) return -1;
			}
		}
	}
}